declare type JSValue = string | number | bigint | boolean | symbol | Function | Array<any> | undefined | null;
declare type JSType = 'string' | 'number' | 'bigint' | 'boolean' | 'symbol' | 'function' | 'object' | 'any' | 'array';
declare type ResolveFn = ((value: any, parent: InputObject, root: InputObject) => JSValue);
interface Schema {
    type?: JSType | JSType[];
    items?: Schema;
    default?: JSValue;
    resolve?: ResolveFn;
    properties?: {
        [key: string]: Schema;
    };
    title?: string;
    description?: string;
    $schema?: string;
}
interface InputObject {
    [key: string]: any;
    $schema?: Schema;
    $resolve?: ResolveFn;
}
declare type InputValue = InputObject | JSValue;

declare function resolveSchema(obj: InputObject, preserveResolve?: boolean): Schema;

declare function generateTypes(schema: Schema, name?: string): string;

declare function applyDefaults(ref: InputObject, input: InputObject): {};

export { InputObject, InputValue, JSType, JSValue, ResolveFn, Schema, applyDefaults, generateTypes, resolveSchema };
